#! /usr/bin/perl 
#This program is used to generate edge information for modularity clustering.
#
# Last Modified: SEP 26, 2012
###########################################################################
# usage: perl gentree.pl -t distancefile -d epsilon -n $numnode -k 0
#	Parameters:
#		-t distance file
#       -d \epsilon, parameter to form a graph
#       -n numnode, number of nodes
#   Output:
#       edge file feed to modularity program
#   Example:
#       perl gentree.pl -t ../test/test.ndist -d 0.04 -n 110
###########################################################################

use strict;
use warnings;
use diagnostics;
#use threads;
my ($inputdis,$pre,$dist,$output);
my $numnode=0;
my $allRequired = 1;
my $size = scalar @ARGV;
for(my $a = 0; $a < $size; $a++)
{
	if($ARGV[$a] eq "-t")
	{
		$a++;
		$allRequired--;
		$inputdis = $ARGV[$a];
		my $ind=rindex($inputdis,"\.");
		$pre=substr($inputdis,0,$ind);
	}elsif($ARGV[$a] eq "-d")
	{
		$a++;
		$dist=$ARGV[$a];
	}elsif($ARGV[$a] eq "-n")
	{
		$a++;
		$numnode=$ARGV[$a];
	}
	elsif($ARGV[$a] eq "-o")
	{
		$a++;
		$output=$ARGV[$a];
	}
	
	
}

unless (defined($output))
{
	$output=$pre.".tree";
}
my @allnode=();
for (my $i=0; $i<$numnode; $i++)
{
	$allnode[$i]=0;
}
open(FILE1,$inputdis) or die "can't open $inputdis";
open(FILE2,">$output") or die "can't open $output";
my @tree=<FILE1>;
my $mindist=100;
for (my $i=0; $i<=$#tree; $i++)
{
	my $line=$tree[$i];
	my @line=split(/\s+/,$line);
	my $dist1=int(($dist-$line[2])*100);
	if ($dist1<$mindist && $line[2]<=$dist)
	{
		$mindist=$dist1;
	}
}
if ($mindist <=0) #or it will increase dist 1 by 1;
{
	$mindist=1;
}

$mindist=1; #turn off the mindist function

for (my $i=0; $i<=$#tree; $i++)
{
	my $line=$tree[$i];
	my @line=split(/\s+/,$line);
    if ($line[2]<=$dist)
	{
		my $a=$line[0];
		my $b=$line[1];
		my $dist1=&roundint(($dist-$line[2])*100)-$mindist+1;
		if ($dist1>0)
		{
			$allnode[$line[0]]=1; #the node has been selected.
			$allnode[$line[1]]=1;
			print FILE2 "$a $b $dist1\n";
		}
	}
	
}
for (my $i=0; $i<$numnode; $i++)
{
	if ($allnode[$i] ==0) #not used in edges
	{
		my $tmp=$numnode+1;
		print FILE2 "$i $tmp 1\n";
	}
}

close FILE1;
close FILE2;

sub roundint { #input: int1. output:roundint
	my ($num)=@_;
	my $roundint=0;
	if ($num-int($num)<0.5)
	{
		$roundint=int($num);
	}else
	{
		$roundint=int($num)+1;
	}
	return $roundint;
}

